/*
 *  tank
 *
 *  Import and Playlist Daemon for autoradio project
 *
 *
 *  Copyright (C) 2017-2018 Christian Pointner <equinox@helsinki.at>
 *
 *  This file is part of tank.
 *
 *  tank is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  tank is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with tank. If not, see <http://www.gnu.org/licenses/>.
 */

'use strict';

var main = main || {};

/***************** models  *****************/

/****** groupList ******/

var groups = null;

main.GroupList = function() {
  this.$this = $(this);
  this.groups = {};
};

main.GroupList.prototype.error = function(error) {
  this.last_error = error;
  this.$this.trigger('error');
};

main.GroupList.prototype.fetch = function() {
  var self = this;
  self.last_error = null;

  $.getJSON( "/api/v1/groups", function(data) {
    self.groups = [];
    for(var key in data.results) {
      self.groups[data.results[key].name] = new main.Group(data.results[key]);
    }
    self.$this.trigger('update');
  }).fail(function(jqxhr, textStatus, error) {
    if(error == "") {
      self.error("connection problem");
    } else {
      self.error(error);
    }
  });
};

main.GroupList.prototype.new_group = function(group_name) {
  var self = this;
  self.last_error = null;

  $.ajax({
    method: "POST",
    url: "/api/v1/groups/" + group_name,
    success: function(data) { self.fetch() },
    dataType: "json"
  }).fail(function(jqxhr, textStatus, error) {
    if(error == "") {
      self.error("connection problem");
    } else {
      self.error(jqxhr.responseJSON.error);
    }
  });
}

/****** group ******/

var group = null;

main.Group = function(inst) {
  this.$this = $(this);
  this.name = inst.name;
  this.created = inst.created;
  this.updated = inst.updated;
  this.files = [];
  this.playlists = [];
};

main.Group.prototype.error = function(error) {
  this.last_error = error;
  this.$this.trigger('error');
};

main.Group.prototype.fetch_files = function() {
  var self = this;
  self.last_error = null;

  $.getJSON("/api/v1/groups/" + self.name + "/files", function(data) {
    self.files = [];
    for(var key in data.results) {
      self.files.push(new main.File(self, data.results[key]));
    }
    self.$this.trigger('update-files');
  }).fail(function(jqxhr, textStatus, error) {
    if(error == "") {
      self.error("connection problem");
    } else {
      self.error(error);
    }
  });
};

main.Group.prototype.fetch_playlists = function() {
  var self = this;
  self.last_error = null;

  $.getJSON("/api/v1/groups/" + self.name + "/playlists", function(data) {
    self.playlists = [];
    for(var key in data.results) {
      self.playlists.push(new main.Playlist(self, data.results[key]));
    }
    self.$this.trigger('update-playlists');
  }).fail(function(jqxhr, textStatus, error) {
    if(error == "") {
      self.error("connection problem");
    } else {
      self.error(error);
    }
  });
};

main.Group.prototype.add_file = function(file_uri) {
  var self = this;
  self.last_error = null;

  $.ajax({
    method: "POST",
    url: "/api/v1/groups/" + self.name + "/files",
    data: JSON.stringify({'source-uri': file_uri}),
    success: function(data) { self.fetch_files() },
    dataType: "json"
  }).fail(function(jqxhr, textStatus, error) {
    if(error == "") {
      self.error("connection problem");
    } else {
      self.error(jqxhr.responseJSON.error);
    }
  });
}

/****** file ******/

main.File = function(group, inst) {
  this.$group = group;
  this.id = inst.id;
  this.created = inst.created;
  this.updated = inst.updated;
  this.source = inst.source;
  this.metadata = inst.metadata;
};

main.File.prototype.delete = function() {
  var self = this;

  $.ajax({
    method: "DELETE",
    url: "/api/v1/groups/" + this.$group.name + "/files/" + this.id,
    success: function(data) { self.$group.fetch_files() },
    dataType: "json"
  }).fail(function(jqxhr, textStatus, error) {
    alert(jqxhr.responseJSON.error + "\n" + JSON.stringify(jqxhr.responseJSON.details, null, 2));
  });
};

/****** playlist ******/

main.Playlist = function(group, inst) {
  this.$group = group;
  this.id = inst.id;
  this.created = inst.created;
  this.updated = inst.updated;
  this.entries = inst.entries;
};

main.Playlist.prototype.delete = function() {
  var self = this;

  $.ajax({
    method: "DELETE",
    url: "/api/v1/groups/" + this.$group.name + "/playlists/" + this.id,
    success: function(data) { self.$group.fetch_playlists() },
    dataType: "json"
  }).fail(function(jqxhr, textStatus, error) {
    alert(jqxhr.responseJSON.error + "\n" + JSON.stringify(jqxhr.responseJSON.details, null, 2));
  });
};

/***************** views  *****************/

/****** groupList  ******/

var groupListView = null;

main.GroupListView = function(model) {
  this.model = model;

  var self = this;
  $(this.model).on('update', function() {
    self.render_list();
  });
  $(this.model).on('error', function() {
    self.render_error();
  });
};

main.GroupListView.prototype.render_list = function() {
  $('#error').hide();

  var list = $('#groupSelect');
  $('option', list).remove();
  list.append($('<option>', {value:'__none__', text:'choose...'}))

  for (var name in this.model.groups) {
    list.append($('<option>', {value:name, text:name}));
  }
};

main.GroupListView.prototype.render_error = function() {
  $('#error-message').text('Error fetching group listing: ' + this.model.last_error)
  $('#error').show();
};

/****** group ******/

var groupView = null;

main.GroupView = function(model) {
  this.model = model;
  this.$el = $('#hiddenTemplates .groupTemplate').clone().removeClass('groupTemplate');

  var self = this;
  var selectFiles = this.$el.find('a.groupSelectFiles');
  var selectPlaylists = this.$el.find('a.groupSelectPlaylists');
  var cardBody = self.$el.find('div.card-body');

  selectFiles.on('click', function() {
    selectFiles.addClass('active');
    selectPlaylists.removeClass('active');
    self.model.fetch_files();
    cardBody.text("... loading files ...");
  });
  selectPlaylists.on('click', function() {
    selectPlaylists.addClass('active');
    selectFiles.removeClass('active');
    self.model.fetch_playlists();
    cardBody.text("... loading playlists ...")
  });

  $(this.model).on('update-files', function() {
    self.render_files(cardBody);
  });
  $(this.model).on('update-playlists', function() {
    self.render_playlists(cardBody);
  });
  $(this.model).on('error', function() {
    self.render_error();
  });

  $('#groupView').html(this.$el);
};

main.GroupView.prototype.render_files = function(body) {
  $('#error').hide();

  var fileListView = new main.FileListView(this.model.files, this);
	fileListView.render();
  body.html(fileListView.$el);
};

main.GroupView.prototype.render_playlists = function(body) {
  $('#error').hide();
  var fileListView = new main.PlaylistListView(this.model.playlists, this);
	fileListView.render();
  body.html(fileListView.$el);
};

main.GroupView.prototype.render_error = function() {
  $('#error-message').text(this.model.last_error)
  $('#error').show();
};

/****** files ******/

main.FileListView = function(model) {
  this.model = model;
  this.$el = $('#hiddenTemplates .filesTemplate').clone().removeClass('filesTemplate');
};

main.FileListView.prototype.render = function() {
  var table = this.$el.find("tbody");
  for (var i = 0; i < this.model.length; i++) {
    var fileView = new main.FileView(this.model[i], this);
    fileView.render();
    table.append(fileView.$el);
  }
};

main.FileView = function(model) {
  this.model = model;
  this.$el = $('#hiddenTemplates .fileTemplate tr').clone();
};

main.FileView.prototype.render = function() {
  this.$el.find('.fileId').text(this.model.id)
  this.$el.find('.fileArtist').text(this.model.metadata.artist)
  this.$el.find('.fileAlbum').text(this.model.metadata.album)
  this.$el.find('.fileTitle').text(this.model.metadata.title)
  this.$el.find('.fileCreated').text(this.model.created)

  var self = this;
  // TODO: implement file edit
  this.$el.find('.fileDelete').on('click', function() {
    self.model.delete();
  });
};

/****** playlists ******/

main.PlaylistListView = function(model) {
  this.model = model;
  this.$el = $('#hiddenTemplates .playlistsTemplate').clone().removeClass('playlistsTemplate');
};

main.PlaylistListView.prototype.render = function() {
  var table = this.$el.find("tbody");
  for (var i = 0; i < this.model.length; i++) {
    var playlistView = new main.PlaylistView(this.model[i], this);
    playlistView.render();
    table.append(playlistView.$el);
  }
};

main.PlaylistView = function(model) {
  this.model = model;
  this.$el = $('#hiddenTemplates .playlistTemplate tr').clone();
};

main.PlaylistView.prototype.render = function() {
  this.$el.find('.playlistId').text(this.model.id)
  this.$el.find('.playlistLength').text(this.model.entries.length)
  this.$el.find('.playlistCreated').text(this.model.created)

  var self = this;
  // TODO: implement playlist edit
  this.$el.find('.playlistDelete').on('click', function() {
    self.model.delete();
  });
};

/***************** controller  *****************/

function modal_group_new_init() {
  var modal = $('#groupNewModal');
  var groupNewName = modal.find('#groupNewName');
  var groupNewBtn = modal.find('button.groupNew');

  groupNewName.on('keyup', function() {
    if(groupNewName.val().length == 0) {
      groupNewBtn.prop('disabled', true)
    } else {
      groupNewBtn.prop('disabled', false)
    }
  });

  modal.find('#groupNewForm').on('submit', function() {
    groupNewBtn.trigger('click');
    return false;
  });

  modal.off().on('show.bs.modal', function(event) {
    groupNewName.val("");

    groupNewBtn.prop('disabled', true)
    groupNewBtn.off('click').on('click', function() {
      groups.new_group(groupNewName.val());
    });
  });
}

function modal_file_add_init() {
  var modal = $('#fileAddModal');
  var fileAddURI = modal.find('#fileAddURI');
  var fileAddBtn = modal.find('button.fileAdd');

  fileAddURI.on('keyup', function() {
    if(fileAddURI.val().length == 0) {
      fileAddBtn.prop('disabled', true)
    } else {
      fileAddBtn.prop('disabled', false)
    }
  });

  modal.find('#fileAddForm').on('submit', function() {
    fileAddBtn.trigger('click');
    return false;
  });

  modal.on('show.bs.modal', function(event) {
    fileAddURI.val("");

    fileAddBtn.prop('disabled', true)
    fileAddBtn.off('click').on('click', function() {
      group.add_file(fileAddURI.val());
    });

  });
}

function main_init() {
  groups = new main.GroupList();
  groupListView = new main.GroupListView(groups);

  modal_group_new_init();
  modal_file_add_init();

  groups.fetch();
  $('#groupSelect').on('change', function(event) {
    var group_name = event.target.value;
    if(group_name != '__none__') {
      group = groups.groups[group_name];
      groupView = new main.GroupView(group);
      group.fetch_files();
      $('button.groupClone').prop('disabled', false);
      $('button.groupAddFile').removeClass('disabled').attr('data-toggle', 'modal');
      $('button.groupAddPlaylist').removeClass('disabled').attr('data-toggle', 'modal');
    } else {
      $('#groupView').empty();
      $('button.groupClone').prop('disabled', true);
      $('button.groupAddFile').addClass('disabled').removeAttr('data-toggle');
      $('button.groupAddPlaylist').addClass('disabled').removeAttr('data-toggle');
    }
  });
}

/**********************************************/
